
//+------------------------------------------------------------------
//| %r  adaptive smoother divergence
//+------------------------------------------------------------------
//
//

#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers    5
#property indicator_color1     DodgerBlue
#property indicator_color2     Orange
#property indicator_color3     LimeGreen
#property indicator_color4     Red
#property indicator_color5     Red
#property indicator_width3     2
#property indicator_width4     2
#property indicator_width5     2
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern string TimeFrame                 = "Current time frame";
extern int    WprPeriod                 = 35;
extern int    SmoothLength              = 5;
extern int    AdaptPeriod               = 21;
extern bool   drawDivergences           = true;
extern bool   divergenceOnValuesVisible = true;
extern bool   divergenceOnChartVisible  = false;
extern color  divergenceBullishColor    = DodgerBlue;
extern color  divergenceBearishColor    = Orange;
extern string drawLinesIdentificator    = "Wpr adp smoother diverge1";
extern string ___                       = "Divergence Alerts";
extern bool   divergenceAlert           = true;
extern bool   divergenceAlertsMessage   = true;
extern bool   divergenceAlertsSound     = true;
extern bool   divergenceAlertsNotify    = false;
extern bool   divergenceAlertsEmail     = false;
extern string divergenceAlertsSoundName = "alert1.wav";

extern int    levelOs                   = -90;
extern int    levelOb                   = -10;
extern int    arrowSize                 = 1; 

extern bool   alertsOn                  = true;
extern bool   alertsOnCurrent           = false;
extern bool   alertsMessage             = true;
extern bool   alertsSound               = true;
extern bool   alertsEmail               = true;
extern bool   alertsNotify              = false;

//
//
//
//
//

double bullishDivergence[];
double bearishDivergence[];
double wpr[];
double trend[];
double wprDa[];
double wprDb[];
double slope[];

//
//
//
//
//

string shortName;
string labelNames;
string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;
double pipMultiplier = 1;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
// 
//
//
//
//

int init()
{
    IndicatorBuffers(7);
    SetIndexBuffer(0,bullishDivergence); SetIndexStyle(0,DRAW_ARROW,0,arrowSize); SetIndexArrow(0,233);
    SetIndexBuffer(1,bearishDivergence); SetIndexStyle(1,DRAW_ARROW,0,arrowSize); SetIndexArrow(1,234);  
    SetIndexBuffer(2,wpr);
    SetIndexBuffer(3,wprDa); 
    SetIndexBuffer(4,wprDb);
    SetIndexBuffer(5,trend);
    SetIndexBuffer(6,slope);
    SetLevelValue(0,0);
    SetLevelValue(1,levelOs);
    SetLevelValue(2,levelOb);
    
    //
    //
    //
    //
    //
   
    timeFrame  = stringToTimeFrame(TimeFrame);   
    labelNames = "Wpr_DivergenceLine "+drawLinesIdentificator+":";
    shortName  = timeFrameToString(timeFrame)+"  Wpr Adaptive Smoother ("+WprPeriod+")";
    IndicatorShortName(shortName);
   
    //
    //
    //
    //
    //
   
    indicatorFileName = WindowExpertName();
    calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
    returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
return(0);
}

//
//
//
//
//

int deinit()
{
   int length=StringLen(labelNames);
   for(int i=ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i);
      if(StringSubstr(name,0,length) == labelNames)  ObjectDelete(name);   
   }
   return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars)  { bullishDivergence[0] = limit+1; return(0); }
         if (Digits==3 || Digits==5) 
               pipMultiplier = 10;
         else  pipMultiplier = 1; 

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      if (slope[limit]==-1) CleanPoint(limit,wprDa,wprDb);
      for (int i=limit; i>=0; i--)
      {
         double dev = iStdDev(NULL,0,AdaptPeriod,0,MODE_SMA,PRICE_CLOSE,i);
         double avg = iSma(dev,AdaptPeriod,i,0);
         if (dev!=0) 
                double period = SmoothLength*avg/dev;
         else          period = SmoothLength; 
         if (period<3) period = 3;
         
         //
         //
         //
         //
         //
         
         double hi   = High[iHighest(NULL,0, MODE_HIGH,WprPeriod,i)];
         double lo   =  Low[iLowest(NULL, 0, MODE_LOW, WprPeriod,i)];
         if (hi!=lo)      
              wpr[i] = iSmooth(-100*(hi-Close[i]) / (hi-lo),period,i,0);
         else wpr[i] = iSmooth(0                           ,period,i,0);  
            trend[i] = trend[i+1];
            slope[i] = slope[i+1]; 
            wprDa[i] = EMPTY_VALUE;
            wprDb[i] = EMPTY_VALUE;
 
              if (wpr[i]>levelOs)  trend[i] =  1;
              if (wpr[i]<levelOb)  trend[i] = -1;
              if (wpr[i]>wpr[i+1]) slope[i] =  1;
              if (wpr[i]<wpr[i+1]) slope[i] = -1;
              if (slope[i] == -1)  PlotPoint(i,wprDa,wprDb,wpr);
              
              if (drawDivergences)
              {
                CatchBullishDivergence(i);
                CatchBearishDivergence(i);
              }         
      
      }
      manageAlerts();
    return(0);
    }
    
    //
    //
    //
    //
    //       
    
    limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
    if (slope[limit]==-1) CleanPoint(limit,wprDa,wprDb);
    for(i=limit; i>=0; i--)
    {
       int y = iBarShift(NULL,timeFrame,Time[i]);
          wpr[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",WprPeriod,SmoothLength,AdaptPeriod,drawDivergences,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,drawLinesIdentificator,"",divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsNotify,divergenceAlertsEmail,divergenceAlertsSoundName,2,y);
          trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",WprPeriod,SmoothLength,AdaptPeriod,drawDivergences,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,drawLinesIdentificator,"",divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsNotify,divergenceAlertsEmail,divergenceAlertsSoundName,5,y);
          slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",WprPeriod,SmoothLength,AdaptPeriod,drawDivergences,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,drawLinesIdentificator,"",divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsNotify,divergenceAlertsEmail,divergenceAlertsSoundName,6,y);
          bullishDivergence[i] = EMPTY_VALUE;
          bearishDivergence[i] = EMPTY_VALUE;
          wprDa[i] = EMPTY_VALUE;
          wprDb[i] = EMPTY_VALUE;
             
          int firstBar = iBarShift(NULL,0,iTime(NULL,timeFrame,y));
          if (i==firstBar)
          {
             bullishDivergence[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",WprPeriod,SmoothLength,AdaptPeriod,drawDivergences,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,drawLinesIdentificator,"",divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsNotify,divergenceAlertsEmail,divergenceAlertsSoundName,0,y);
             bearishDivergence[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",WprPeriod,SmoothLength,AdaptPeriod,drawDivergences,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,drawLinesIdentificator,"",divergenceAlert,divergenceAlertsMessage,divergenceAlertsSound,divergenceAlertsNotify,divergenceAlertsEmail,divergenceAlertsSoundName,1,y);
          }
   } 
   for(i=limit; i>=0; i--) if (slope[i] == -1)  PlotPoint(i,wprDa,wprDb,wpr);
   manageAlerts();  
return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workSmooth[][5];
double iSmooth(double price, double length, int r, int instanceNo=0)
{
   if (ArrayRange(workSmooth,0)!=Bars) ArrayResize(workSmooth,Bars); instanceNo *= 5; r = Bars-r-1;
 	if(r<=2) { workSmooth[r][instanceNo] = price; workSmooth[r][instanceNo+2] = price; workSmooth[r][instanceNo+4] = price; return(price); }
   
   //
   //
   //
   //
   //
   
	double alpha = 0.45*(length-1.0)/(0.45*(length-1.0)+2.0);
   	  workSmooth[r][instanceNo+0] =  price+alpha*(workSmooth[r-1][instanceNo]-price);
	     workSmooth[r][instanceNo+1] = (price - workSmooth[r][instanceNo])*(1-alpha)+alpha*workSmooth[r-1][instanceNo+1];
	     workSmooth[r][instanceNo+2] =  workSmooth[r][instanceNo+0] + workSmooth[r][instanceNo+1];
	     workSmooth[r][instanceNo+3] = (workSmooth[r][instanceNo+2] - workSmooth[r-1][instanceNo+4])*MathPow(1.0-alpha,2) + MathPow(alpha,2)*workSmooth[r-1][instanceNo+3];
	     workSmooth[r][instanceNo+4] =  workSmooth[r][instanceNo+3] + workSmooth[r-1][instanceNo+4]; 
   return(workSmooth[r][instanceNo+4]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

double workSma[][2];
double iSma(double price, int period, int r, int instanceNo=0)
{
   if (ArrayRange(workSma,0)!= Bars) ArrayResize(workSma,Bars); instanceNo *= 2; r = Bars-r-1;

   //
   //
   //
   //
   //
      
   workSma[r][instanceNo] = price;
   if (r>=period)
          workSma[r][instanceNo+1] = workSma[r-1][instanceNo+1]+(workSma[r][instanceNo]-workSma[r-period][instanceNo])/period;
   else { workSma[r][instanceNo+1] = 0; for(int k=0; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo];  
          workSma[r][instanceNo+1] /= k; }
   return(workSma[r][instanceNo+1]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CatchBullishDivergence(int shift)
{
   shift++;
         bullishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"l"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"l"+"os" + DoubleToStr(Time[shift],0));            
   if(!IsIndicatorLow(shift)) return;  

   //
   //
   //
   //
   //
      
   int currentLow = shift;
   int lastLow    = GetIndicatorLastLow(shift+1);
    
   if(wpr[currentLow] > wpr[lastLow] && Low[currentLow] < Low[lastLow])
   {
      bullishDivergence[currentLow] = wpr[currentLow]-Point*pipMultiplier;
      if(divergenceOnChartVisible)  DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],    divergenceBullishColor,STYLE_SOLID);
      if(divergenceOnValuesVisible) DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],wpr[currentLow],wpr[lastLow],divergenceBullishColor,STYLE_SOLID);
      if(divergenceAlert)           DisplayAlert("Classical bullish divergence",currentLow); 
   }
     
   //
   //
   //
   //
   //
        
   if(wpr[currentLow] < wpr[lastLow] && Low[currentLow] > Low[lastLow])
   {
      bullishDivergence[currentLow] = wpr[currentLow]-Point*pipMultiplier;
      if(divergenceOnChartVisible)  DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],    divergenceBullishColor,STYLE_DOT);
      if(divergenceOnValuesVisible) DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],wpr[currentLow],wpr[lastLow],divergenceBullishColor,STYLE_DOT);
      if(divergenceAlert)           DisplayAlert("Reverse bullish divergence",currentLow); 
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CatchBearishDivergence(int shift)
{
   shift++; 
         bearishDivergence[shift] = EMPTY_VALUE;
            ObjectDelete(labelNames+"h"+DoubleToStr(Time[shift],0));
            ObjectDelete(labelNames+"h"+"os" + DoubleToStr(Time[shift],0));            
   if(IsIndicatorPeak(shift) == false) return;
   
   //
   //
   //
   //
   //
   
   int currentPeak = shift;
   int lastPeak    = GetIndicatorLastPeak(shift+1);
      
   if(wpr[currentPeak] < wpr[lastPeak] && High[currentPeak]>High[lastPeak])
   {
       bearishDivergence[currentPeak] = wpr[currentPeak]+Point*pipMultiplier;
       if (divergenceOnChartVisible)  DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],  divergenceBearishColor,STYLE_SOLID);
       if (divergenceOnValuesVisible) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],wpr[currentPeak],wpr[lastPeak],divergenceBearishColor,STYLE_SOLID);
       if (divergenceAlert)           DisplayAlert("Classical bearish divergence",currentPeak); 
   }
   
   //
   //
   //
   //
   //

   if(wpr[currentPeak] > wpr[lastPeak] && High[currentPeak] < High[lastPeak])
   {
      bearishDivergence[currentPeak] = wpr[currentPeak]+Point*pipMultiplier;
      if (divergenceOnChartVisible ) DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],  divergenceBearishColor,STYLE_DOT);
      if (divergenceOnValuesVisible) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],wpr[currentPeak],wpr[lastPeak],divergenceBearishColor,STYLE_DOT);
      if (divergenceAlert)           DisplayAlert("Reverse bearish divergence",currentPeak);
   }   
}

//
//
//
//
//

bool IsIndicatorPeak(int shift)
{
   if(wpr[shift] >= wpr[shift+1] && wpr[shift] > wpr[shift+2] && wpr[shift] > wpr[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

bool IsIndicatorLow(int shift)
{
   if(wpr[shift] <= wpr[shift+1] && wpr[shift] < wpr[shift+2] && wpr[shift] < wpr[shift-1])
       return(true);
   else 
       return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastPeak(int shift)
{
    for(int i = shift+5; i < Bars; i++)
    {
       if(wpr[i] >= wpr[i+1] && wpr[i] > wpr[i+2] && wpr[i] >= wpr[i-1] && wpr[i] > wpr[i-2])
         return(i);
    }
return(-1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int GetIndicatorLastLow(int shift)
{
    for (int i = shift+5; i < Bars; i++)
    {
       if (wpr[i] <= wpr[i+1] && wpr[i] < wpr[i+2] && wpr[i] <= wpr[i-1] && wpr[i] < wpr[i-2])
         return(i);
}
     
return(-1);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void DrawPriceTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
    string label = labelNames+first+"os"+DoubleToStr(t1,0);
    
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, 0, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//
//
//
//
//

void DrawIndicatorTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
   int indicatorWindow = WindowFind(shortName);
   if (indicatorWindow < 0) return;
   string label = labelNames+first+DoubleToStr(t1,0);
   
      ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, indicatorWindow, t1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, 0);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//
//
//
//
//

void DisplayAlert(string doWhat, int shift)
{
    string dmessage;
    static datetime lastAlertTime;
    if(shift <= 2 && Time[0] != lastAlertTime)
    {
      dmessage =  StringConcatenate(Symbol()," at ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Wpr Adaptive Smooth ",doWhat);
          if (divergenceAlertsMessage) Alert(dmessage);
          if (divergenceAlertsEmail)   SendMail(StringConcatenate(Symbol()," Wpr Adaptive Smoother "),dmessage);
          if (divergenceAlertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Wpr Adaptive Smoother " +" "+dmessage));
          if (divergenceAlertsSound)   PlaySound(divergenceAlertsSoundName); 
          lastAlertTime = Time[0];
    }
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
// 
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
// 
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"oversold");
         if (trend[whichBar] ==-1) doAlert(whichBar,"overbought");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Wpr Adaptive Smoother ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Wpr Adaptive Smoother "),message);
          if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," Wpr Adaptive Smoother " +" "+message));
          if (alertsSound)   PlaySound("alert2.wav");
   }
}